'use strict';

angular.module('adf.electre')
        .service('domService', 
            [
                function () {
                    function template () {
                        /**
                         * The dom element to use.
                         * @private
                         */
                        var domElement;

                        /**
                         * Give the 'domElement' the value of 'domElement_'.
                         * @param {Element} domElement_
                         * @returns {template}
                         */
                        function setDom (domElement_) { domElement = domElement_; return this; }
                        /**
                         * Give the 'domElement' object.
                         * @returns {domElement_|Element}
                         */
                        function getDom () { return domElement; }
                        /**
                         * Create a HTML DOM Node with the specified 'tagName'.
                         * @param {String} tagName
                         * @returns {template}
                         */
                        function createElement (tagName) { domElement = document.createElement(tagName); return this; }
                        /**
                         * Donne à l'attribut 'attributeName' la valeur 'attributeValue'.
                         * @param {String} attributeName
                         * @param {String} attributeValue
                         * @returns {template}
                         */
                        function setAttribute (attributeName, attributeValue) {
                            if (attributeName === "id") domElement.id = attributeValue;
                            else if (attributeName === "style") {
                                var style = "";
                                for (var style_name in attributeValue) {
                                    style += style_name + ":" + attributeValue[style_name] + ";";
                                }
                                domElement.style = style;
                            } else domElement.setAttribute(attributeName, attributeValue);
                            return this;
                        }
                        /**
                         * Donne la valeur de l'attribut demandé.
                         * @param {String} attributeName
                         * @returns {String}
                         */
                        function getAttribute (attributeName) { return domElement.getAttribute(attributeName); }
                        /**
                         * Ajoute une class à l'élément.
                         * @param className {String|Array}
                         * @returns {template}
                         */
                        function addClass (className) { domElement.classList.add(className); return this; }

                        /**
                         * Enlève une class de l'élément.
                         * @param className {String|Array}
                         * @returns {template}
                         */
                        function removeClass (className) { domElement.classList.remove(className); return this; }

                        /**
                         * Dit si l'élément à une class spécifique.
                         * @param className {String}
                         * @returns {boolean}
                         */
                        function hasClass (className) { return domElement.classList.contains(className); }
                        /**
                         * Add content
                         * @param content
                         * @returns {template}
                         */
                        function addContent (content) { domElement.innerHTML = content; return this; }

                        /**
                         * Ajoute un Node enfant avant la fin de celui-ci.
                         * @param {Element|Array<Element>} childNode
                         * @returns {Object}
                         */
                        function appendChild (childNode) { domElement.appendChild(childNode); return this; }

                        this.setDom = setDom;
                        this.getDom = getDom;
                        this.createElement = createElement;
                        this.setAttribute = setAttribute;
                        this.getAttribute = getAttribute;
                        this.appendChild = appendChild;
                        this.addClass = addClass;
                        this.removeClass = removeClass;
                        this.hasClass = hasClass;
                        this.addContent = addContent;
                    }

                    template.createDomFromJSON = function (json, tag_name) {
                        var _template = new template();
                        if (tag_name)
                            createDomFromTemplate(json, tag_name, _template);
                        else
                            for (var tag_name in json)
                                createDomFromTemplate(json[tag_name], tag_name, _template);
                        return _template.getDom();
                    };

                    function createDomFromTemplate (template_object, tag_name, template_js) {
                        /* Creating element */
                        template_js.createElement(tag_name);
                        /* Add classes */
                        if (template_object.classes) {
                            if (typeof template_object.classes === "string") template_js.addClass(template_object.classes);
                            else {
                                for (var c = 0 ; c < template_object.classes.length ; c++) {
                                    template_js.addClass(template_object.classes[c]);
                                }
                            }
                        }
                        /* Add attributes */
                        if (template_object.attributes)
                            for (var attr_name in template_object.attributes)
                                template_js.setAttribute(attr_name, template_object.attributes[attr_name]);
                        /* Add content */
                        if (template_object.content) template_js.addContent(template_object.content);
                        /* Add children */
                        if (template_object.children) {
                            for (var children_tag in template_object.children) {
                                if (template_object.children[children_tag].length)
                                    for (var t = 0 ; t < template_object.children[children_tag].length ; t++)
                                        template_js.appendChild(
                                            template.createDomFromJSON(template_object.children[children_tag][t], children_tag)
                                        );
                                else
                                    template_js.appendChild(
                                        template.createDomFromJSON(template_object.children[children_tag], children_tag)
                                    );
                            }
                        }
                    }
                    
                    var template_ = new template();
                    return {
                        createDomFromJSON:template.createDomFromJSON, json2dom:template.createDomFromJSON,

                        setDom:template_.setDom, getDom:template_.getDom,
                        createElement:template_.createElement,
                        setAttribute:template_.setAttribute, getAttribute:template_.getAttribute,
                        appendChild:template_.appendChild
                    };
                }
            ]
        );